import { FederatedMouseEvent, Graphics } from "pixi.js";
import { BACKGROUND_COLOR, PATH_COLOR, STATION_RADIUS, STATION_OUTLINE_WIDTH, UP_SCALE } from "./Constants";
import { useTransform } from "./TransformContext";
import { useCallback, useEffect, useState } from "react";
import { StationData } from "./types";
import { useMapData } from "./MapDataContext";

interface StationProps {
	station: StationData;
}

export function Station({
	station
}: Readonly<StationProps>) {
	const draw = useCallback((g: Graphics) => {
		g.clear();
		g.circle(station.latitude * UP_SCALE, station.longitude * UP_SCALE, STATION_RADIUS);
		g.fill({color: PATH_COLOR});
		g.stroke({color: BACKGROUND_COLOR, width: STATION_OUTLINE_WIDTH});
	}, []);

	return (
		<pixiContainer>
			<pixiGraphics draw={draw}/>
			<StationLabel station={station}/>
		</pixiContainer>
	);
}

export function StationLabel({
	station
}: Readonly<StationProps>) {
	const { rotation, scale } = useTransform();
	const { setStationOffset } = useMapData();

	const [position, setPosition] = useState({ x: station.offset_x, y: station.offset_y });
	const [isDragging, setIsDragging] = useState(false);
    const [startPosition, setStartPosition] = useState({ x: 0, y: 0 });
    const [startMousePosition, setStartMousePosition] = useState({ x: 0, y: 0 });

	if(!station) {
		console.error("station is null");
		return null;
	}

    const handlePointerDown = (e: FederatedMouseEvent) => {
        setIsDragging(true);
        setStartPosition({
            x: position.x,
            y: position.y
        });
		setStartMousePosition({
			x: e.globalX,
			y: e.globalY
		});
		
        e.stopPropagation();
    };
    const handlePointerMove = (e: FederatedMouseEvent) => {
        if (!isDragging) return;
		const dx = (e.globalX - startMousePosition.x) / scale;
		const dy = (e.globalY - startMousePosition.y) / scale;
		const cos = Math.cos(rotation);
		const sin = Math.sin(rotation);
		const newPosition = {
			x: startPosition.x + dx * cos + dy * sin,
			y: startPosition.y - dx * sin + dy * cos
		};
		setPosition(newPosition);
		setStationOffset(station.id, newPosition.x, newPosition.y);
        e.stopPropagation();
    };

    const handlePointerUp = (e: FederatedMouseEvent) => {
        setIsDragging(false);
        e.stopPropagation();
    };

	return (
		<pixiContainer
			eventMode='static'
			interactive
			onPointerDown={handlePointerDown}
			onGlobalPointerMove={handlePointerMove}
			onPointerUp={handlePointerUp}
			onPointerUpOutside={handlePointerUp}
			width={48}
			height={48}
			x={station.latitude * UP_SCALE + position.x}
			y={station.longitude * UP_SCALE + position.y}
		>
			<pixiText
				text={station.name}
				rotation={-rotation}
				style={{
					fontSize: 48,
					fontWeight: 'bold',
					fill: "#ffffff"
				}}
			/>
		</pixiContainer>
	);
}
